import type { Key } from '../crypto';
import type { ConnectionRecord } from '../modules/connections';
import type { ResolvedDidCommService } from '../modules/didcomm';
import type { OutboundTransport } from '../transport/OutboundTransport';
import type { OutboundMessage, OutboundPackage, EncryptedMessage } from '../types';
import type { AgentMessage } from './AgentMessage';
import type { EnvelopeKeys } from './EnvelopeService';
import type { AgentContext } from './context';
import { DID_COMM_TRANSPORT_QUEUE } from '../constants';
import { Logger } from '../logger';
import { DidCommDocumentService } from '../modules/didcomm';
import { DidResolverService } from '../modules/dids/services/DidResolverService';
import { OutOfBandRepository } from '../modules/oob/repository';
import { MessageRepository } from '../storage/MessageRepository';
import { EnvelopeService } from './EnvelopeService';
import { TransportService } from './TransportService';
export interface TransportPriorityOptions {
    schemes: string[];
    restrictive?: boolean;
}
export declare class MessageSender {
    private envelopeService;
    private transportService;
    private messageRepository;
    private logger;
    private didResolverService;
    private didCommDocumentService;
    private outOfBandRepository;
    readonly outboundTransports: OutboundTransport[];
    constructor(envelopeService: EnvelopeService, transportService: TransportService, messageRepository: MessageRepository, logger: Logger, didResolverService: DidResolverService, didCommDocumentService: DidCommDocumentService, outOfBandRepository: OutOfBandRepository);
    registerOutboundTransport(outboundTransport: OutboundTransport): void;
    packMessage(agentContext: AgentContext, { keys, message, endpoint, }: {
        keys: EnvelopeKeys;
        message: AgentMessage;
        endpoint: string;
    }): Promise<OutboundPackage>;
    private sendMessageToSession;
    sendPackage(agentContext: AgentContext, { connection, encryptedMessage, options, }: {
        connection: ConnectionRecord;
        encryptedMessage: EncryptedMessage;
        options?: {
            transportPriority?: TransportPriorityOptions;
        };
    }): Promise<void>;
    sendMessage(agentContext: AgentContext, outboundMessage: OutboundMessage, options?: {
        transportPriority?: TransportPriorityOptions;
    }): Promise<void>;
    sendMessageToService(agentContext: AgentContext, { message, service, senderKey, returnRoute, connectionId, }: {
        message: AgentMessage;
        service: ResolvedDidCommService;
        senderKey: Key;
        returnRoute?: boolean;
        connectionId?: string;
    }): Promise<void>;
    private retrieveServicesByConnection;
}
export declare function isDidCommTransportQueue(serviceEndpoint: string): serviceEndpoint is typeof DID_COMM_TRANSPORT_QUEUE;
