import type { AgentContext } from '../../../../agent';
import type { Attachment } from '../../../../decorators/attachment/Attachment';
import type { FormatAcceptOfferOptions, FormatAcceptProposalOptions, FormatAcceptRequestOptions, FormatAutoRespondCredentialOptions, FormatAutoRespondOfferOptions, FormatAutoRespondProposalOptions, FormatAutoRespondRequestOptions, FormatCreateOfferOptions, FormatCreateOfferReturn, FormatCreateProposalOptions, FormatCreateProposalReturn, FormatCreateRequestOptions, CredentialFormatCreateReturn, FormatProcessCredentialOptions, FormatProcessOptions } from '../CredentialFormatServiceOptions';
import type { JsonLdCredentialFormat } from './JsonLdCredentialFormat';
import { DidResolverService } from '../../../dids/services/DidResolverService';
import { W3cCredentialService } from '../../../vc';
import { CredentialFormatService } from '../CredentialFormatService';
export declare class JsonLdCredentialFormatService extends CredentialFormatService<JsonLdCredentialFormat> {
    private w3cCredentialService;
    private didResolver;
    constructor(w3cCredentialService: W3cCredentialService, didResolver: DidResolverService);
    readonly formatKey: "jsonld";
    readonly credentialRecordType: "w3c";
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the proposed credential
     * @returns object containing associated attachment, formats and filtersAttach elements
     *
     */
    createProposal(agentContext: AgentContext, { credentialFormats }: FormatCreateProposalOptions<JsonLdCredentialFormat>): Promise<FormatCreateProposalReturn>;
    /**
     * Method called on reception of a propose credential message
     * @param options the options needed to accept the proposal
     */
    processProposal(agentContext: AgentContext, { attachment }: FormatProcessOptions): Promise<void>;
    acceptProposal(agentContext: AgentContext, { attachId, credentialFormats, proposalAttachment }: FormatAcceptProposalOptions<JsonLdCredentialFormat>): Promise<FormatCreateOfferReturn>;
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the credential offer
     * @returns object containing associated attachment, formats and offersAttach elements
     *
     */
    createOffer(agentContext: AgentContext, { credentialFormats, attachId }: FormatCreateOfferOptions<JsonLdCredentialFormat>): Promise<FormatCreateOfferReturn>;
    processOffer(agentContext: AgentContext, { attachment }: FormatProcessOptions): Promise<void>;
    acceptOffer(agentContext: AgentContext, { credentialFormats, attachId, offerAttachment }: FormatAcceptOfferOptions<JsonLdCredentialFormat>): Promise<CredentialFormatCreateReturn>;
    /**
     * Create a credential attachment format for a credential request.
     *
     * @param options The object containing all the options for the credential request is derived
     * @returns object containing associated attachment, formats and requestAttach elements
     *
     */
    createRequest(agentContext: AgentContext, { credentialFormats }: FormatCreateRequestOptions<JsonLdCredentialFormat>): Promise<CredentialFormatCreateReturn>;
    processRequest(agentContext: AgentContext, { attachment }: FormatProcessOptions): Promise<void>;
    acceptRequest(agentContext: AgentContext, { credentialFormats, attachId, requestAttachment }: FormatAcceptRequestOptions<JsonLdCredentialFormat>): Promise<CredentialFormatCreateReturn>;
    /**
     * Derive a verification method using the issuer from the given verifiable credential
     * @param credential the verifiable credential we want to sign
     * @return the verification method derived from this credential and its associated issuer did, keys etc.
     */
    private deriveVerificationMethod;
    /**
     * Processes an incoming credential - retrieve metadata, retrieve payload and store it in the Indy wallet
     * @param options the issue credential message wrapped inside this object
     * @param credentialRecord the credential exchange record for this credential
     */
    processCredential(agentContext: AgentContext, { credentialRecord, attachment, requestAttachment }: FormatProcessCredentialOptions): Promise<void>;
    private compareCredentialSubject;
    private compareProofs;
    supportsFormat(format: string): boolean;
    deleteCredentialById(): Promise<void>;
    areCredentialsEqual: (message1: Attachment, message2: Attachment) => boolean;
    shouldAutoRespondToProposal(agentContext: AgentContext, { offerAttachment, proposalAttachment }: FormatAutoRespondProposalOptions): boolean;
    shouldAutoRespondToOffer(agentContext: AgentContext, { offerAttachment, proposalAttachment }: FormatAutoRespondOfferOptions): boolean;
    shouldAutoRespondToRequest(agentContext: AgentContext, { offerAttachment, requestAttachment }: FormatAutoRespondRequestOptions): boolean;
    shouldAutoRespondToCredential(agentContext: AgentContext, { credentialAttachment, requestAttachment }: FormatAutoRespondCredentialOptions): boolean;
}
