"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonLdCredentialFormatService = void 0;
const tsyringe_1 = require("tsyringe");
const error_1 = require("../../../../error");
const JsonTransformer_1 = require("../../../../utils/JsonTransformer");
const MessageValidator_1 = require("../../../../utils/MessageValidator");
const objEqual_1 = require("../../../../utils/objEqual");
const DidDocument_1 = require("../../../dids/domain/DidDocument");
const DidResolverService_1 = require("../../../dids/services/DidResolverService");
const vc_1 = require("../../../vc");
const models_1 = require("../../../vc/models");
const CredentialFormatSpec_1 = require("../../models/CredentialFormatSpec");
const CredentialFormatService_1 = require("../CredentialFormatService");
const JsonLdCredentialOptions_1 = require("./JsonLdCredentialOptions");
const JSONLD_VC_DETAIL = 'aries/ld-proof-vc-detail@v1.0';
const JSONLD_VC = 'aries/ld-proof-vc@1.0';
let JsonLdCredentialFormatService = class JsonLdCredentialFormatService extends CredentialFormatService_1.CredentialFormatService {
    constructor(w3cCredentialService, didResolver) {
        super();
        this.formatKey = 'jsonld';
        this.credentialRecordType = 'w3c';
        this.areCredentialsEqual = (message1, message2) => {
            // FIXME: this implementation doesn't make sense. We can't loop over stringified objects...
            const obj1 = message1.getDataAsJson();
            const obj2 = message2.getDataAsJson();
            return objEqual_1.deepEqual(obj1, obj2);
        };
        this.w3cCredentialService = w3cCredentialService;
        this.didResolver = didResolver;
    }
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the proposed credential
     * @returns object containing associated attachment, formats and filtersAttach elements
     *
     */
    async createProposal(agentContext, { credentialFormats }) {
        const format = new CredentialFormatSpec_1.CredentialFormatSpec({
            format: JSONLD_VC_DETAIL,
        });
        const jsonLdFormat = credentialFormats.jsonld;
        if (!jsonLdFormat) {
            throw new error_1.AriesFrameworkError('Missing jsonld payload in createProposal');
        }
        const jsonLdCredential = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(jsonLdFormat);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredential);
        // jsonLdFormat is now of type SignCredentialOptionsRFC0593
        const attachment = this.getFormatData(jsonLdFormat, format.attachId);
        return { format, attachment };
    }
    /**
     * Method called on reception of a propose credential message
     * @param options the options needed to accept the proposal
     */
    async processProposal(agentContext, { attachment }) {
        const credProposalJson = attachment.getDataAsJson();
        if (!credProposalJson) {
            throw new error_1.AriesFrameworkError('Missing jsonld credential proposal data payload');
        }
        const messageToValidate = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(credProposalJson);
        MessageValidator_1.MessageValidator.validateSync(messageToValidate);
    }
    async acceptProposal(agentContext, { attachId, credentialFormats, proposalAttachment }) {
        // if the offer has an attachment Id use that, otherwise the generated id of the formats object
        const format = new CredentialFormatSpec_1.CredentialFormatSpec({
            attachId,
            format: JSONLD_VC_DETAIL,
        });
        const jsonLdFormat = credentialFormats === null || credentialFormats === void 0 ? void 0 : credentialFormats.jsonld;
        if (jsonLdFormat) {
            // if there is an offer, validate
            const jsonLdCredentialOffer = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(jsonLdFormat);
            MessageValidator_1.MessageValidator.validateSync(jsonLdCredentialOffer);
        }
        const credentialProposal = proposalAttachment.getDataAsJson();
        const jsonLdCredentialProposal = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(credentialProposal);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredentialProposal);
        const offerData = jsonLdFormat !== null && jsonLdFormat !== void 0 ? jsonLdFormat : credentialProposal;
        const attachment = this.getFormatData(offerData, format.attachId);
        return { format, attachment };
    }
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the credential offer
     * @returns object containing associated attachment, formats and offersAttach elements
     *
     */
    async createOffer(agentContext, { credentialFormats, attachId }) {
        // if the offer has an attachment Id use that, otherwise the generated id of the formats object
        const format = new CredentialFormatSpec_1.CredentialFormatSpec({
            attachId,
            format: JSONLD_VC_DETAIL,
        });
        const jsonLdFormat = credentialFormats === null || credentialFormats === void 0 ? void 0 : credentialFormats.jsonld;
        if (!jsonLdFormat) {
            throw new error_1.AriesFrameworkError('Missing jsonld payload in createOffer');
        }
        const jsonLdCredential = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(jsonLdFormat);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredential);
        const attachment = this.getFormatData(jsonLdFormat, format.attachId);
        return { format, attachment };
    }
    async processOffer(agentContext, { attachment }) {
        const credentialOfferJson = attachment.getDataAsJson();
        if (!credentialOfferJson) {
            throw new error_1.AriesFrameworkError('Missing jsonld credential offer data payload');
        }
        const jsonLdCredential = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(credentialOfferJson);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredential);
    }
    async acceptOffer(agentContext, { credentialFormats, attachId, offerAttachment }) {
        const jsonLdFormat = credentialFormats === null || credentialFormats === void 0 ? void 0 : credentialFormats.jsonld;
        const credentialOffer = offerAttachment.getDataAsJson();
        const requestData = jsonLdFormat !== null && jsonLdFormat !== void 0 ? jsonLdFormat : credentialOffer;
        const jsonLdCredential = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(requestData);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredential);
        const format = new CredentialFormatSpec_1.CredentialFormatSpec({
            attachId,
            format: JSONLD_VC_DETAIL,
        });
        const attachment = this.getFormatData(requestData, format.attachId);
        return { format, attachment };
    }
    /**
     * Create a credential attachment format for a credential request.
     *
     * @param options The object containing all the options for the credential request is derived
     * @returns object containing associated attachment, formats and requestAttach elements
     *
     */
    async createRequest(agentContext, { credentialFormats }) {
        const jsonLdFormat = credentialFormats === null || credentialFormats === void 0 ? void 0 : credentialFormats.jsonld;
        const format = new CredentialFormatSpec_1.CredentialFormatSpec({
            format: JSONLD_VC_DETAIL,
        });
        if (!jsonLdFormat) {
            throw new error_1.AriesFrameworkError('Missing jsonld payload in createRequest');
        }
        const jsonLdCredential = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(jsonLdFormat);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredential);
        const attachment = this.getFormatData(jsonLdFormat, format.attachId);
        return { format, attachment };
    }
    async processRequest(agentContext, { attachment }) {
        const requestJson = attachment.getDataAsJson();
        if (!requestJson) {
            throw new error_1.AriesFrameworkError('Missing jsonld credential request data payload');
        }
        const jsonLdCredential = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(requestJson);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredential);
    }
    async acceptRequest(agentContext, { credentialFormats, attachId, requestAttachment }) {
        var _a, _b;
        const jsonLdFormat = credentialFormats === null || credentialFormats === void 0 ? void 0 : credentialFormats.jsonld;
        // sign credential here. credential to be signed is received as the request attachment
        // (attachment in the request message from holder to issuer)
        const credentialRequest = requestAttachment.getDataAsJson();
        const credentialData = jsonLdFormat !== null && jsonLdFormat !== void 0 ? jsonLdFormat : credentialRequest;
        const jsonLdCredential = new JsonLdCredentialOptions_1.JsonLdCredentialDetail(credentialData);
        MessageValidator_1.MessageValidator.validateSync(jsonLdCredential);
        const verificationMethod = (_b = (_a = credentialFormats === null || credentialFormats === void 0 ? void 0 : credentialFormats.jsonld) === null || _a === void 0 ? void 0 : _a.verificationMethod) !== null && _b !== void 0 ? _b : (await this.deriveVerificationMethod(agentContext, credentialData.credential, credentialRequest));
        if (!verificationMethod) {
            throw new error_1.AriesFrameworkError('Missing verification method in credential data');
        }
        const format = new CredentialFormatSpec_1.CredentialFormatSpec({
            attachId,
            format: JSONLD_VC,
        });
        const options = credentialData.options;
        if (options.challenge || options.domain || options.credentialStatus) {
            throw new error_1.AriesFrameworkError('The fields challenge, domain and credentialStatus not currently supported in credential options ');
        }
        const verifiableCredential = await this.w3cCredentialService.signCredential(agentContext, {
            credential: JsonTransformer_1.JsonTransformer.fromJSON(credentialData.credential, models_1.W3cCredential),
            proofType: credentialData.options.proofType,
            verificationMethod: verificationMethod,
        });
        const attachment = this.getFormatData(verifiableCredential, format.attachId);
        return { format, attachment };
    }
    /**
     * Derive a verification method using the issuer from the given verifiable credential
     * @param credential the verifiable credential we want to sign
     * @return the verification method derived from this credential and its associated issuer did, keys etc.
     */
    async deriveVerificationMethod(agentContext, credential, credentialRequest) {
        // extract issuer from vc (can be string or Issuer)
        let issuerDid = credential.issuer;
        if (typeof issuerDid !== 'string') {
            issuerDid = issuerDid.id;
        }
        // this will throw an error if the issuer did is invalid
        const issuerDidDocument = await this.didResolver.resolveDidDocument(agentContext, issuerDid);
        // find first key which matches proof type
        const proofType = credentialRequest.options.proofType;
        // actually gets the key type(s)
        const keyType = this.w3cCredentialService.getVerificationMethodTypesByProofType(proofType);
        if (!keyType || keyType.length === 0) {
            throw new error_1.AriesFrameworkError(`No Key Type found for proofType ${proofType}`);
        }
        const verificationMethod = await DidDocument_1.findVerificationMethodByKeyType(keyType[0], issuerDidDocument);
        if (!verificationMethod) {
            throw new error_1.AriesFrameworkError(`Missing verification method for key type ${keyType}`);
        }
        return verificationMethod.id;
    }
    /**
     * Processes an incoming credential - retrieve metadata, retrieve payload and store it in the Indy wallet
     * @param options the issue credential message wrapped inside this object
     * @param credentialRecord the credential exchange record for this credential
     */
    async processCredential(agentContext, { credentialRecord, attachment, requestAttachment }) {
        const credentialAsJson = attachment.getDataAsJson();
        const credential = JsonTransformer_1.JsonTransformer.fromJSON(credentialAsJson, models_1.W3cVerifiableCredential);
        MessageValidator_1.MessageValidator.validateSync(credential);
        // compare stuff in the proof object of the credential and request...based on aca-py
        const requestAsJson = requestAttachment.getDataAsJson();
        const request = JsonTransformer_1.JsonTransformer.fromJSON(requestAsJson, JsonLdCredentialOptions_1.JsonLdCredentialDetail);
        if (Array.isArray(credential.proof)) {
            // question: what do we compare here, each element of the proof array with the request???
            throw new error_1.AriesFrameworkError('Credential arrays are not supported');
        }
        else {
            // do checks here
            this.compareCredentialSubject(credential, request.credential);
            this.compareProofs(credential.proof, request.options);
        }
        const verifiableCredential = await this.w3cCredentialService.storeCredential(agentContext, {
            credential: credential,
        });
        credentialRecord.credentials.push({
            credentialRecordType: this.credentialRecordType,
            credentialRecordId: verifiableCredential.id,
        });
    }
    compareCredentialSubject(credential, request) {
        if (!objEqual_1.deepEqual(credential.credentialSubject, request.credentialSubject)) {
            throw new error_1.AriesFrameworkError('Received credential subject does not match subject from credential request');
        }
    }
    compareProofs(credentialProof, requestProof) {
        if (credentialProof.domain !== requestProof.domain) {
            throw new error_1.AriesFrameworkError('Received credential proof domain does not match domain from credential request');
        }
        if (credentialProof.challenge !== requestProof.challenge) {
            throw new error_1.AriesFrameworkError('Received credential proof challenge does not match challenge from credential request');
        }
        if (credentialProof.type !== requestProof.proofType) {
            throw new error_1.AriesFrameworkError('Received credential proof type does not match proof type from credential request');
        }
        if (credentialProof.proofPurpose !== requestProof.proofPurpose) {
            throw new error_1.AriesFrameworkError('Received credential proof purpose does not match proof purpose from credential request');
        }
    }
    supportsFormat(format) {
        const supportedFormats = [JSONLD_VC_DETAIL, JSONLD_VC];
        return supportedFormats.includes(format);
    }
    async deleteCredentialById() {
        throw new Error('Not implemented.');
    }
    shouldAutoRespondToProposal(agentContext, { offerAttachment, proposalAttachment }) {
        return this.areCredentialsEqual(proposalAttachment, offerAttachment);
    }
    shouldAutoRespondToOffer(agentContext, { offerAttachment, proposalAttachment }) {
        return this.areCredentialsEqual(proposalAttachment, offerAttachment);
    }
    shouldAutoRespondToRequest(agentContext, { offerAttachment, requestAttachment }) {
        return this.areCredentialsEqual(offerAttachment, requestAttachment);
    }
    shouldAutoRespondToCredential(agentContext, { credentialAttachment, requestAttachment }) {
        const credentialAsJson = credentialAttachment.getDataAsJson();
        const credential = JsonTransformer_1.JsonTransformer.fromJSON(credentialAsJson, models_1.W3cVerifiableCredential);
        if (Array.isArray(credential.proof)) {
            throw new error_1.AriesFrameworkError('Credential arrays are not supported');
        }
        else {
            // do checks here
            try {
                const requestAsJson = requestAttachment.getDataAsJson();
                const request = JsonTransformer_1.JsonTransformer.fromJSON(requestAsJson, JsonLdCredentialOptions_1.JsonLdCredentialDetail);
                this.compareCredentialSubject(credential, request.credential);
                this.compareProofs(credential.proof, request.options);
                return true;
            }
            catch (error) {
                return false;
            }
        }
    }
};
JsonLdCredentialFormatService = __decorate([
    tsyringe_1.injectable(),
    __metadata("design:paramtypes", [vc_1.W3cCredentialService, DidResolverService_1.DidResolverService])
], JsonLdCredentialFormatService);
exports.JsonLdCredentialFormatService = JsonLdCredentialFormatService;
//# sourceMappingURL=JsonLdCredentialFormatService.js.map