import type { JsonObject, JsonValue } from '../../types';
import type { SingleOrArray } from '../../utils/type';
import type { GetProofsOptions, GetProofsResult, GetTypeOptions } from './models';
export declare type JsonLdDoc = Record<string, unknown>;
export interface VerificationMethod extends JsonObject {
    id: string;
    [key: string]: JsonValue;
}
export interface Proof extends JsonObject {
    verificationMethod: string | VerificationMethod;
    [key: string]: JsonValue;
}
export interface DocumentLoaderResult {
    contextUrl?: string | null;
    documentUrl: string;
    document: Record<string, unknown>;
}
export declare type DocumentLoader = (url: string) => Promise<DocumentLoaderResult>;
export declare const orArrayToArray: <T>(val?: SingleOrArray<T> | undefined) => T[];
export declare const _includesContext: (options: {
    document: JsonLdDoc;
    contextUrl: string;
}) => boolean;
/**
 * Gets a supported linked data proof from a JSON-LD Document
 * Note - unless instructed not to the document will be compacted
 * against the security v2 context @see https://w3id.org/security/v2
 *
 * @param options Options for extracting the proof from the document
 *
 * @returns {GetProofsResult} An object containing the matched proofs and the JSON-LD document
 */
export declare const getProofs: (options: GetProofsOptions) => Promise<GetProofsResult>;
/**
 * Formats an input date to w3c standard date format
 * @param date {number|string} Optional if not defined current date is returned
 *
 * @returns {string} date in a standard format as a string
 */
export declare const w3cDate: (date?: string | number | undefined) => string;
/**
 * Gets the JSON-LD type information for a document
 * @param document {any} JSON-LD document to extract the type information from
 * @param options {GetTypeInfoOptions} Options for extracting the JSON-LD document
 *
 * @returns {object} Type info for the JSON-LD document
 */
export declare const getTypeInfo: (document: JsonObject, options: GetTypeOptions) => Promise<{
    types: string[];
    alias: string;
}>;
